function [D,dt]=hledejkapky(jm)
%[D,dt]=hledejkapky(jm)
%Snazi se identifikovat svetle kapicky zaznamenane na videu.
%"jm" je jmeno mpg souboru (videa) s natocenym pohybem kapicek
%"D" je matice se zaznamenanymi kapickami (svetlymi oblastmi):
%  Kazdy sloupec odpovida jedne detekci kapicky (na jednom snimku).
%  Na prvnim radku je cislo snimku, na kterem byla kapicka zaznamenana,
%  na druhem radku vertikalni pozice kapicky, na tretim horizontalni pozice,
%  na ctvrtem radku je intenzita signalu kapicky.
%"dt" je casovy krok mezi dvema snimky.
%
%Z kazdeho snimku odecte prumerny obrazek. P
%Pak projde kazdy snimek a hleda, jestli v nejake ctvercove oblasti s velikosti (2*k+1 pixelu)^2
% je prumerny signal alespon "pomer" - krat a zaroven alespon o "delta"
% vetsi nez prumerny signal ve vetsi ctvercove oblasti s velikosti (2*o+1 pixelu)^2
%Kapka, ktera je blize nez "o2" k jine, silnejsi, kapce, se ignoruje.



%Parametry vyhodnocovani:
k=2;
o=10;
o2=o;
pomer=3;
delta=8;

%Zakladni kontrola, jestli nedelame uplne blbosti:
if o<=k
 error('Okoli kapky (k) musi byt vetsi nez kapka (o).');
end
if pomer <=1
 error('pomer musi byt vetsi nez 1. ');
end

%Nacteni dat:
A=VideoReader(jm);
n=A.NumFrames;
tc=A.Duration;
for j=1:n,
 B=read(A,j);
 V(:,:,j)=mean(B,3);
end

Dark=mean(V,3);
%figure; graffoto(Dark); pause(1e-3);
for j=1:n,
 V(:,:,j)=V(:,:,j)-Dark;
end

%Horodolni otoceni:
%for j=1:n,
% V(:,:,j)=V(end:(-1):1,:,j);
%end

%Grafeni prumerneho obrazku a volba zpracovavane oblasti:
F1=figure; graffoto(Dark); pause(1e-3);
r=input('Zadej vyhodnocovanou oblast ');
if length(r)>0
 V=V(r(1):r(2),r(3):r(4),:);
 Dark=Dark(r(1):r(2),r(3):r(4),:);
end
figure(F1); graffoto(Dark); title('Prumerny obrazek'); pause(1e-3);
[a,b,c]=size(V);


figure; graffoto(V(:,:,1)); title('Prvni obrazek'); pause(1e-3);
%Hledani kapek:
D=[];
for jjj=1:n,
 jjj/n
 D1=[];
 prumer=mean(mean(V(:,:,jjj)));
 prumer=min([prumer 0]);
 for j=(o+1):(a-o),
  for jj=(o+1):(b-o),
   Kp=V((j-k):(j+k),(jj-k):(jj+k),jjj)-prumer;
   Op=V((j-o):(j+o),(jj-o):(jj+o),jjj)-prumer;
   kp=mean(mean(Kp));
   op=mean(mean(Op));
   if op>=0
    hranicenas=op*pomer;
   else
    hranicenas=op/pomer;
   end
   if kp>hranicenas & kp>=op+delta
    pridej=[jjj; j; jj; kp];
    D1=[D1 pridej];
   end
  end
 end

 D2=[];
 [nica,m,nicc]=size(D1);
 if m>0
  [nic,por]=sort(D1(4,:));
  D1=D1(:,por(end:(-1):1));
  while m>0,
   D2=[D2 D1(:,1)]; D1=D1(:,2:end); D3=[];
   for q=1:length(D1(1,:)),
    xy=sqrt((D1(2,q)-D2(2,end))^2 + (D1(3,q)-D2(3,end))^2);
    if xy>o2
     D3=[D3 D1(:,q)];
    end
   end
   D1=D3;
   [nica,m,nicc]=size(D1);
  end
  D=[D D2];
 end
end

Dcas=D;
Dcas(1,:)=(Dcas(1,:)-1)*tc/(n-1);
dt=tc/(n-1);

%Grafeni vysledku:
grafkapky(Dcas);

end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function graffoto(A)
r=size(A);
rs=r(2);
rr=r(1);
pcolor(1:rs,1:rr,A); shading interp;
axis equal;
view(0,90);
%set(gca,'YDir','reverse'); %http://www.mathworks.com/help/matlab/ref/axes_props.html
%set(gca,'XAxisLocation','top');
xlabel('horizontalni pixely (2. souradnice)');
ylabel('vertikalni pixely (1. souradnice)');
colorbar;
axis([1 rs 1 rr]);
end
